"""
loctocat is the easiest way to add OAuth 2.0 device flow authentication to your Python apps.

https://loctocat.celsiusnarhwal.dev
"""

from loctocat.auth import *
from loctocat.exceptions import *
from loctocat.predefined import *

__version__ = "1.0.0"
